@@ -41,6 +41,8 @@
 #include <getopt.h>
 #include <pwd.h>
 #include <regex.h>
+#include <ctype.h>
+#include <assert.h>
 
 #ifdef WITH_SELINUX
 #include <selinux/selinux.h>
@@ -50,14 +52,36 @@
 #include "comm.h"
 #include "signals.h"
 
-
 #define PROC_BASE "/proc"
 #define MAX_NAMES (int)(sizeof(unsigned long)*8)
 
+#define TSECOND "s"
+#define TMINUTE "m"
+#define THOUR   "h"
+#define TDAY    "d" 
+#define TWEEK   "w"
+#define TMONTH  "M" 
+#define TYEAR   "y"
+
+#define TMAX_SECOND 31536000
+#define TMAX_MINUTE 525600  
+#define TMAX_HOUR   8760    
+#define TMAX_DAY    365     
+#define TMAX_WEEK   48      
+#define TMAX_MONTH  12      
+#define TMAX_YEAR   1       
+
+#define ER_REGFAIL -1
+#define ER_NOMEM   -2
+#define ER_UNKWN   -3
+#define ER_OOFRA   -4
+
+#define NOT_PIDOF_OPTION if (pidof) usage(NULL)
 
 static int verbose = 0, exact = 0, interactive = 0, reg = 0,
            quiet = 0, wait_until_dead = 0, process_group = 0,
            ignore_case = 0, pidof;
+static long younger_than = 0, older_than = 0;
 
 static int
 ask (char *name, pid_t pid, const int signal)
@@ -95,6 +119,66 @@ ask (char *name, pid_t pid, const int signal)
   /* Never should get here */
 }
 
+static double
+uptime()
+{
+   char * savelocale;
+   char buf[2048];
+   FILE* file;
+   if (!(file=fopen( PROC_BASE "/uptime", "r"))) {
+      fprintf(stderr, "error opening uptime file\n");	
+      exit(1);
+   }
+   savelocale = setlocale(LC_NUMERIC, NULL);
+   setlocale(LC_NUMERIC,"C");
+   fscanf(file, "%s", buf);
+   fclose(file);
+   setlocale(LC_NUMERIC,savelocale);
+   return atof(buf);
+}
+
+/* process age from jiffies to seconds via uptime */
+static double process_age(const unsigned jf)
+{
+   double sc_clk_tck = sysconf(_SC_CLK_TCK);
+   assert(sc_clk_tck > 0);
+   return uptime() - jf / sc_clk_tck;
+}
+
+/* returns requested time interval in seconds, 
+ negative indicates error has occured  
+ */
+static long
+parse_time_units(const char* age)
+{
+   char *unit;
+   long num;
+
+   num = strtol(age,&unit,10);
+   if (age == unit) /* no digits found */
+     return -1;
+   if (unit[0] == '\0') /* no units found */
+     return -1;
+
+   switch(unit[0]) {
+   case 's':
+     return num;
+   case 'm':
+     return (num * 60);
+   case 'h':
+     return (num * 60 * 60);
+   case 'd':
+     return (num * 60 * 60 * 24);
+   case 'w':
+     return (num * 60 * 60 * 24 * 7);
+   case 'M':
+     return (num * 60 * 60 * 24 * 7 * 4);
+   case 'y':
+     return (num * 60 * 60 * 24 * 7 * 4 * 12);
+   }
+   return -1;
+}
+
 static int
 match_process_uid(pid_t pid, uid_t uid)
 {
@@ -254,7 +338,7 @@ kill_all (int signal, int names, char **namelist, struct passwd *pwent)
     {
       pid_t id;
       int found_name = -1;
-
+      double process_age_sec = 0;
       /* match by UID */
       if (pwent && match_process_uid(pid_table[i], pwent->pw_uid)==0)
 	continue;
@@ -281,9 +365,24 @@ kill_all (int signal, int names, char **namelist, struct passwd *pwent)
 	}
       free (path);
       okay = fscanf (file, "%*d (%15[^)]", comm) == 1;
-      (void) fclose (file);
-      if (!okay)
+      if (!okay) {
+	fclose(file);
 	continue;
+      }
+      if ( younger_than || older_than ) {
+	 rewind(file);
+	 unsigned int proc_stt_jf = 0;
+	 okay = fscanf(file, "%*d %*s %*s %*s %*s %*s %*s %*s %*s %*s %*s %*s %*s %*s %*s %*s %*s %*s %*s %*s %*s %ull", 
+		       &proc_stt_jf) == 1;
+	 if (!okay) {
+	    fclose(file);
+	    continue;
+	 }
+	 process_age_sec = process_age(proc_stt_jf);
+	 assert(process_age_sec > 0);
+      }
+      (void) fclose (file);
+       
       got_long = 0;
       command = NULL;		/* make gcc happy */
       length = strlen (comm);
@@ -356,6 +455,11 @@ kill_all (int signal, int names, char **namelist, struct passwd *pwent)
 	    }
 	  else /* non-regex */
 	    {
+	      if ( younger_than && process_age_sec && (process_age_sec > younger_than ) )
+		 continue;
+	      if ( older_than   && process_age_sec && (process_age_sec < older_than ) )
+		 continue;
+	       
  	      if (!sts[j].st_dev)
 	        {
 	          if (length != COMM_LEN - 1 || name_len[j] < COMM_LEN - 1)
@@ -516,8 +620,10 @@ usage_pidof (void)
 
 
 static void
-usage_killall (void)
+usage_killall (const char *msg)
 {
+  if (msg != NULL)
+    fprintf(stderr, "%s\n", msg);
 #ifdef WITH_SELINUX
    fprintf(stderr, _(
      "Usage: killall [-Z CONTEXT] [-u USER] [ -eIgiqrvw ] [ -SIGNAL ] NAME...\n"));
@@ -531,6 +637,8 @@ usage_killall (void)
     "  -e,--exact          require exact match for very long names\n"
     "  -I,--ignore-case    case insensitive process name match\n"
     "  -g,--process-group  kill process group instead of process\n"
+    "  -y,--younger-than   kill processes younger than TIME\n"
+    "  -o,--older-than     kill processes older than TIME\n"		    
     "  -i,--interactive    ask for confirmation before killing\n"
     "  -l,--list           list all known signal names\n"
     "  -q,--quiet          don't print complaints\n"
@@ -550,12 +658,12 @@ usage_killall (void)
 
 
 static void
-usage (void)
+usage (const char *msg)
 {
   if (pidof)
     usage_pidof ();
   else
-    usage_killall ();
+    usage_killall (msg);
   exit (1);
 }
 
@@ -580,13 +688,17 @@ main (int argc, char **argv)
   int myoptind;
   struct passwd *pwent = NULL;
   struct stat isproc;
-  
+  char yt[16];
+  char ot[16];
+
   //int optsig = 0;
 
   struct option options[] = {
     {"exact", 0, NULL, 'e'},
     {"ignore-case", 0, NULL, 'I'},
     {"process-group", 0, NULL, 'g'},
+    {"younger-than", 1, NULL, 'y'},
+    {"older-than", 1, NULL, 'o'},
     {"interactive", 0, NULL, 'i'},
     {"list-signals", 0, NULL, 'l'},
     {"quiet", 0, NULL, 'q'},
@@ -611,7 +723,7 @@ main (int argc, char **argv)
   security_context_t scontext = NULL;
   regex_t scontext_reg;
 
-  if ( argc < 2 ) usage(); /* do the obvious thing... */
+  if ( argc < 2 ) usage(NULL); /* do the obvious thing... */
 #endif /*WITH_SELINUX*/
 
   name = strrchr (*argv, '/');
@@ -625,93 +737,97 @@ main (int argc, char **argv)
 
   opterr = 0;
 #ifdef WITH_SELINUX
-  while ( (optc = getopt_long_only(argc,argv,"egilqrs:u:vwZ:VI",options,NULL)) != EOF) {
+  while ( (optc = getopt_long_only(argc,argv,"egy:o:ilqrs:u:vwZ:VI",options,NULL)) != -1) {
 #else
-  while ( (optc = getopt_long_only(argc,argv,"egilqrs:u:vwVI",options,NULL)) != EOF) {
+  while ( (optc = getopt_long_only(argc,argv,"egy:o:ilqrs:u:vwVI",options,NULL)) != -1) {
 #endif
     switch (optc) {
-      case 'e':
-        exact = 1;
-        break;
-      case 'g':
-        process_group = 1;
-        break;
-      case 'i':
-        if (pidof)
-          usage();
-        interactive = 1;
-        break;
-      case 'l':
-        if (pidof)
-          usage();
-        list_signals();
-        return 0;
-        break;
-      case 'q':
-        if (pidof)
-          usage();
-        quiet = 1;
-        break;
-      case 'r':
-	if (pidof)
-	  usage();
-	reg = 1;
-	break;
-      case 's':
-	sig_num = get_signal (optarg, "killall");
-        break;
-      case 'u':
-	if (pidof)
-	  usage();
-        if (!(pwent = getpwnam(optarg)))
-	  {
-            fprintf (stderr, _("Cannot find user %s\n"), optarg);
-            exit (1);
-          }
-        break;
-      case 'v':
-        if (pidof)
-          usage();
-        verbose = 1;
-        break;
-      case 'w':
-        if (pidof)
-          usage();
-        wait_until_dead = 1;
-        break;
-      case 'I':
-        ignore_case = 1;
-        break;
-      case 'V':
-        print_version();
-        return 0;
-        break;
+    case 'e':
+      exact = 1;
+      break;
+    case 'g':
+      process_group = 1;
+      break;
+    case 'y':
+      NOT_PIDOF_OPTION;
+      strncpy(yt, optarg, 16);
+      if ( 0 >= (younger_than = parse_time_units(yt) ) )
+	    usage(_("Invalid time format"));
+      break;
+    case 'o':
+      NOT_PIDOF_OPTION;
+      strncpy(ot, optarg, 16);
+      if ( 0 >= (older_than = parse_time_units(ot) ) )
+	    usage(_("Invalid time format"));
+      break;
+    case 'i':
+      NOT_PIDOF_OPTION;
+      interactive = 1;
+      break;
+    case 'l':
+      NOT_PIDOF_OPTION;
+      list_signals();
+      return 0;
+      break;
+    case 'q':
+      NOT_PIDOF_OPTION;
+      quiet = 1;
+      break;
+    case 'r':
+      NOT_PIDOF_OPTION;
+	  reg = 1;
+	  break;
+    case 's':
+	  sig_num = get_signal (optarg, "killall");
+      break;
+    case 'u':
+      NOT_PIDOF_OPTION;
+      if (!(pwent = getpwnam(optarg))) {
+        fprintf (stderr, _("Cannot find user %s\n"), optarg);
+        exit (1);
+      }
+      break;
+    case 'v':
+      NOT_PIDOF_OPTION;
+      verbose = 1;
+      break;
+    case 'w':
+      NOT_PIDOF_OPTION;
+      wait_until_dead = 1;
+      break;
+    case 'I':
+      ignore_case = 1;
+      break;
+    case 'V':
+      print_version();
+      return 0;
+      break;
 #ifdef WITH_SELINUX
-      case 'Z': 
-        if (is_selinux_enabled()>0) {
-	  scontext=optarg;
-          if (regcomp(&scontext_reg, scontext, REG_EXTENDED|REG_NOSUB) != 0) {
-            fprintf(stderr, _("Bad regular expression: %s\n"), scontext);
-            exit (1);
-	  }
-        } else 
-           fprintf(stderr, "Warning: -Z (--context) ignored. Requires an SELinux enabled kernel\n");
-        break;
+    case 'Z': 
+      if (is_selinux_enabled()>0) {
+	    scontext=optarg;
+        if (regcomp(&scontext_reg, scontext, REG_EXTENDED|REG_NOSUB) != 0) {
+          fprintf(stderr, _("Bad regular expression: %s\n"), scontext);
+          exit (1);
+	    }
+      } else 
+        fprintf(stderr, "Warning: -Z (--context) ignored. Requires an SELinux enabled kernel\n");
+      break;
 #endif /*WITH_SELINUX*/
-      case '?':
-        /* Signal names are in uppercase, so check to see if the argv
-         * is upper case */
-        if (argv[optind-1][1] >= 'A' && argv[optind-1][1] <= 'Z') {
-	      sig_num = get_signal (argv[optind-1]+1, "killall");
+    case '?':
+      /* Signal names are in uppercase, so check to see if the argv
+       * is upper case */
+      if (argv[optind-1][1] >= 'A' && argv[optind-1][1] <= 'Z') {
+	    sig_num = get_signal (argv[optind-1]+1, "killall");
+      } else {
+        /* Might also be a -## signal too */
+        if (argv[optind-1][1] >= '0' && argv[optind-1][1] <= '9') {
+          sig_num = atoi(argv[optind-1]+1);
         } else {
-          /* Might also be a -## signal too */
-          if (argv[optind-1][1] >= '0' && argv[optind-1][1] <= '9') {
-            sig_num = atoi(argv[optind-1]+1);
-          } else {
-            usage();
-          }
+          usage(NULL);
         }
-        break;
+      }
+      break;
     }
   }
   myoptind = optind;
@@ -720,18 +836,16 @@ main (int argc, char **argv)
 #else
   if ((argc - myoptind < 1) && pwent==NULL)	  
 #endif
-    usage();
+    usage(NULL);
 
-  if (argc - myoptind > MAX_NAMES + 1)
-    {
-      fprintf (stderr, _("Maximum number of names is %d\n"), MAX_NAMES);
-      exit (1);
-    }
-  if (stat("/proc/self/stat", &isproc)==-1)
-    {
-      fprintf (stderr, _("%s is empty (not mounted ?)\n"), PROC_BASE);
-      exit (1);
-    }
+  if (argc - myoptind > MAX_NAMES + 1) {
+    fprintf (stderr, _("Maximum number of names is %d\n"), MAX_NAMES);
+    exit (1);
+  }
+  if (stat("/proc/self/stat", &isproc)==-1) {
+    fprintf (stderr, _("%s is empty (not mounted ?)\n"), PROC_BASE);
+    exit (1);
+  }
   argv = argv + myoptind;
   /*printf("sending signal %d to procs\n", sig_num);*/
 #ifdef WITH_SELINUX
